Scalar

Struct Scalar 

pub struct Scalar(/* private fields */);
Available on crate feature arithmetic only.
Scalars are elements in the finite field modulo n.

Trait impls

Much of the important functionality is provided by traits from the ff crate:

  • Field represents elements of finite fields and provides:
  • PrimeField represents elements of prime fields and provides:
    • from_repr/to_repr for converting field elements from/to big integers.
    • MULTIPLICATIVE_GENERATOR and ROOT_OF_UNITY constants.

Please see the documentation for the relevant traits for more information.

Implementations

Source

impl Scalar

pub const ZERO: Self

Zero scalar.

pub const ONE: Self

Multiplicative identity.

pub fn to_bytes(&self) -> FieldBytes

Returns the SEC1 encoding of this scalar.

pub const fn add(&self, rhs: &Self) -> Self

Returns self + rhs mod n

pub const fn double(&self) -> Self

Returns 2*self.

pub const fn sub(&self, rhs: &Self) -> Self

Returns self - rhs mod n.

pub const fn multiply(&self, rhs: &Self) -> Self

Returns self * rhs mod n

pub const fn square(&self) -> Self

Returns self * self mod n

pub const fn shr_vartime(&self, shift: u32) -> Scalar

Right shifts the scalar.

Note: not constant-time with respect to the shift parameter.

pub fn invert(&self) -> CtOption<Self>

Compute scalar inversion: 1 / self.

pub fn invert_vartime(&self) -> CtOption<Self>

Compute scalar inversion: 1 / self in variable-time.

pub const fn pow_vartime<const RHS_LIMBS: usize>( &self, exp: &Uint<RHS_LIMBS>, ) -> Self

Returns self^exp, where exp is a little-endian integer exponent.

This operation is variable time with respect to the exponent exp.

If the exponent is fixed, this operation is constant time.

pub const fn sqn_vartime(&self, n: usize) -> Self

Returns self^(2^n) mod p.

This operation is variable time with respect to the exponent n.

If the exponent is fixed, this operation is constant time.

pub fn is_odd(&self) -> Choice

Is integer representing equivalence class odd?

pub fn is_even(&self) -> Choice

Is integer representing equivalence class even?